#'Wind power density on s2dv_cube objects
#'
#'@author Llorenç Lledó, \email{llledo@bsc.es}
#'@description Wind Power Density computes the wind power that is available for extraction per square meter of swept area. 
#'@description It is computed as 0.5*ro*wspd^3. As this function is non-linear, it will give inaccurate results if used with period means.
#'
#'@param wind a s2dv_cube object with instantaneous wind speeds expressed in m/s obtained from CST_Load or s2dv_cube functions from CSTools pacakge
#'@param ro a scalar, or alternatively a multidimensional array with the same dimensions as wind, with the air density expressed in kg/m^3. By default it takes the value 1.225, the standard density of air at 15ºC and 1013.25 hPa.
#'@return A s2dv_cube object containing Wind Power Density expressed in W/m^2.
#'
#'@examples
#'wind <- array(rweibull(n = 100, shape = 2, scale = 6), c(member = 10, lat = 2, lon = 5))
#'wind <- CSTools::s2dv_cube(data = wind, lat = c(40, 41), lon = 1:5,
#'                  Variable = list(varName = 'sfcWind', level = 'Surface'), 
#'                  Datasets = 'synthetic', when = Sys.time(),
#'                  Dates = list(start = '1990-01-01 00:00:00', end = '1990-01-01 00:00:00'),
#'                  source_file = NA)
#'WPD <- CST_WindPowerDensity(wind)
#'
#'@export
CST_WindPowerDensity <- function(wind, ro = 1.225) {
  if (!inherits(wind, 's2dv_cube')) {
    stop("Parameter 'wind' must be of the class 's2dv_cube', ",
         "as output by CSTools::CST_Load.")
  }
  wind$data <- WindPowerDensity(wind$data, ro = ro)
  if ('Variable' %in% names(wind)) {
    if ('varName' %in% names(wind$Variable)) {
      wind$Variable$varName <- 'WindPowerDensity'
    }
  }
  return(wind)
}

#'Wind power density on multidimensional array objects
#'
#'@author Llorenç Lledó, \email{llledo@bsc.es}
#'@description Wind Power Density computes the wind power that is available for extraction per square meter of swept area. 
#'@description It is computed as 0.5*ro*wspd^3. As this function is non-linear, it will give inaccurate results if used with period means.
#'
#'@param wind a multidimensional array, vector or scalar with instantaneous wind speeds expressed in m/s.
#'@param ro a scalar, or alternatively a multidimensional array with the same dimensions as wind, with the air density expressed in kg/m^3. By default it takes the value 1.225, the standard density of air at 15ºC and 1013.25 hPa.
#'@return An array with the same dimensions as wind, containing Wind Power Density expressed in W/m^2.
#'
#'@examples
#'wind <- rweibull(n = 100, shape = 2, scale = 6)
#'WPD <- WindPowerDensity(wind)
#'
#'@export
WindPowerDensity <- function(wind, ro = 1.225) {
	return(0.5 * ro * wind^3)
}
